/**
Atomix is a remake of the eponym game from Atari ST / Amiga
The program is developped under an event that took place on internet with the goal to make a game in a weekend
Copyright (C) 2014  Alexandre "LittleWhite" LAURENT

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

website: http://www.developpez.net/forums/d1360495/applications/developpement-2d-3d-jeux/projets/we-jeux-3-Atomix/
e-mail: lw.demoscene@gmail.com
**/


#include "Game.hpp"

#include "NEngine/NEngine.h"
#include "NEngine/Window.h"
#include "NEngine/Renderer.h"
#include "NEngine/Time.h"
#include "NEngine/SpriteLoader.h"
#include "NEngine/InputManager.h"
#include "NEngine/SoundEngine.h"
#include "NEngine/Exceptions/FileNotFoundException.h"
#include "NEngine/Types/Colour.h"

#include "SDL/SDL.h"

#include "UI/BoardGame.hpp"
#include "common.hpp"

#include <cassert>

Game::Game(NE::NEngine* pEngine)
    :m_pEngine(pEngine),
     m_fpsNumber(0),m_fpsCounter(0),m_fpsLastUpdateTime(0),m_lastUpdateTime(0),
     m_lastDropTime(0),
     m_lastShootTime(0),
     m_isRunning(true),
     m_resLoader(pEngine->getSpriteLoader()),
     m_state(GS_TITLE),m_pBoardGame(NULL),m_pCursor(NULL)
{
    assert(pEngine);
}

Game::~Game()
{
    for (std::vector<Piece*>::iterator itPiece = m_pieces.begin() ; itPiece != m_pieces.end() ;
         ++itPiece)
    {
        delete (*itPiece);
    }

    for (std::vector<Bomb*>::iterator itBomb = m_bombs.begin() ; itBomb != m_bombs.end() ;
         ++itBomb)
    {
        delete (*itBomb);
    }
    delete m_pCursor;
    delete m_pBoardGame;
}

bool Game::init()
{
    // Magenta pixels as transparancy (as often in old games)
    m_pEngine->getSpriteLoader()->setTransparencyColour(Colour(255,0,255));
    
    return true;
}

bool Game::load()
{
    try
    {
        // Loading generic sprites
        m_resLoader.getSprite(BOARDGAME_BORDER_PATH);
        m_resLoader.getSprite(BOARDGAME_LINE_PATH);
        m_resLoader.getSprite(BOARDGAME_CITY_PATH);

        m_resLoader.getSprite(BOARDGAME_SCORE_PATH);
        m_resLoader.getSprite(BOARDGAME_LEVEL_PATH);
        m_resLoader.getSprite(BOARDGAME_BLOCKS_PATH);

        m_resLoader.getSprite(BOARDGAME_BOMBS_PATH);
        m_resLoader.getSprite(BOARDGAME_SPEED_PATH);
        m_resLoader.getSprite(BOARDGAME_TIME_PATH);
        m_resLoader.getSprite(BOARDGAME_BONUS_PATH);

        m_resLoader.getSprite(CURSOR_PATH);

        m_resLoader.getSprite(BLOCKS_TILE_PATH);
        m_resLoader.getSprite(BLOCK_EXPLOSION_PATH);

        m_resLoader.getSprite(BOMB_PATH);
        m_resLoader.getSprite(EXPLOSION_PATH);

        m_resLoader.getSprite(NEWGAME_PATH);
        m_resLoader.getSprite(QUIT_PATH);
        m_resLoader.getSprite(GAMEOVER_PATH);

        m_resLoader.loadFont(FONT_PATH,USize2(FONT_WIDTH,FONT_HEIGHT),' ');
    }
    catch (std::exception& e)
    {
        NEError << e.what() + std::string("\n");
        return false;
    }
    /*
    // Load musics
    bool bResult = true;
    bool bMusicResult = true;
    
    try
    {
        m_pEngine->getSoundEngine()->load(MUSIC_PATH);
    }
    catch (FileNotFoundException e)
    {
        NEError << e.what() << "\n";
        bMusicResult = false;
    }
    
    bResult &= m_pEngine->getSoundEngine()->load(MOVE_SOUND_PATH);
    bResult &= m_pEngine->getSoundEngine()->load(COLLISION_SOUND_PATH);
    if ( bResult == false )
    {
        return false;
    }*/
    m_pBoardGame = new BoardGame(m_resLoader.getSprite(BOARDGAME_BORDER_PATH), m_resLoader.getSprite(BOARDGAME_LINE_PATH),m_resLoader.getSprite(BOARDGAME_CITY_PATH),
                                 m_resLoader.getSprite(BOARDGAME_SCORE_PATH),m_resLoader.getSprite(BOARDGAME_LEVEL_PATH),m_resLoader.getSprite(BOARDGAME_BLOCKS_PATH),
                                 m_resLoader.getSprite(BOARDGAME_BOMBS_PATH),m_resLoader.getSprite(BOARDGAME_SPEED_PATH),m_resLoader.getSprite(BOARDGAME_TIME_PATH),
                                 m_resLoader.getSprite(BOARDGAME_BONUS_PATH),
                                 m_resLoader.getFont(FONT_PATH));
    m_pCursor = new Cursor(m_resLoader.getSprite(CURSOR_PATH));

    if ( m_pBoardGame == NULL || m_pCursor == NULL)
    {
        return false;
    }
    
    return true;
}

void Game::play()
{
    bool bResult = true;

    // Main loop
    while ( m_isRunning && // If the game is still active
            m_pEngine->getInputManager()->needEscape() == false &&  // If no button to quit have been used
            m_pEngine->getWindow()->needWindowClosure() == false &&  // or window cross
            bResult == true ) // or a problem occured
    {
        // Display game
        bResult &= this->render();
        
        // This solve a problem in Windows, because the events have to be updated in the same thread than the video.
        unsigned int actualTime = m_pEngine->getTime()->getTime();
        //if ( actualTime - m_lastUpdateTime > 25 ) // All 25ms
        {
            bResult &= this->update(actualTime - m_lastUpdateTime);
            m_lastUpdateTime = m_pEngine->getTime()->getTime();
        }

        // FPS management
        m_fpsCounter++;
        if ( actualTime - m_fpsLastUpdateTime > 1000 ) // After one second
        {            
            m_fpsNumber= m_fpsCounter;
            m_fpsCounter = 0;
            m_fpsLastUpdateTime = m_pEngine->getTime()->getTime();

            NELog(0) << "FPS: " << m_fpsNumber << "\n";
        }
    }
}

bool Game::render()
{
    unsigned int currentTime = m_pEngine->getTime()->getTime();

    if ( m_state != Game::GS_GAMEOVER )
    {
        m_pEngine->getRenderer()->clearScreen(Colour(0,0,0,255));
    }
    
    if ( m_state == Game::GS_TITLE )
    {
        m_pBoardGame->render(*m_pEngine->getRenderer(),m_gameInfo);

        m_pEngine->getRenderer()->drawSurface(IVec2(WINDOW_WIDTH/2-(m_resLoader.getSprite(NEWGAME_PATH))->getSize().width/2,
                                   WINDOW_HEIGHT/2-(m_resLoader.getSprite(NEWGAME_PATH))->getSize().height/2 - 100),m_resLoader.getSprite(NEWGAME_PATH));

        m_pEngine->getRenderer()->drawSurface(IVec2(WINDOW_WIDTH/2-(m_resLoader.getSprite(QUIT_PATH))->getSize().width/2,
                                   WINDOW_HEIGHT/2-(m_resLoader.getSprite(QUIT_PATH))->getSize().height/2 + 100),m_resLoader.getSprite(QUIT_PATH));

        m_pCursor->render(*m_pEngine->getRenderer());
    }
    else if ( m_state == Game::GS_GAME)
    {
        m_pBoardGame->render(*m_pEngine->getRenderer(),m_gameInfo);

        // Display blocks
        for (std::vector<Piece*>::iterator itPiece = m_pieces.begin() ; itPiece != m_pieces.end() ;
             ++itPiece)
        {
            (*itPiece)->render(*m_pEngine->getRenderer(),m_resLoader.getSprite(BLOCKS_TILE_PATH),currentTime);
        }

        // Display bombs
        for (std::vector<Bomb*>::iterator itBomb = m_bombs.begin() ; itBomb != m_bombs.end() ;
             ++itBomb)
        {
            (*itBomb)->render(*m_pEngine->getRenderer(),currentTime);
        }

        m_pCursor->render(*m_pEngine->getRenderer());
    }
    else if ( m_state == GS_GAMEOVER )
    {
        m_pEngine->getRenderer()->drawSurface(IVec2(WINDOW_WIDTH/2-(m_resLoader.getSprite(GAMEOVER_PATH))->getSize().width/2,
                                   WINDOW_HEIGHT/2-(m_resLoader.getSprite(GAMEOVER_PATH))->getSize().height/2),m_resLoader.getSprite(GAMEOVER_PATH));

        m_pCursor->render(*m_pEngine->getRenderer());
    }
    
    // Render something here :D
    if ( m_pEngine->getRenderer()->updateWindow() == false )
    {
        NEError << "Fail to draw on the screen\n";
    }

    return true;
}

bool Game::update(unsigned deltaTime)
{
    m_pEngine->getInputManager()->update();

    unsigned int currentTime = m_pEngine->getTime()->getTime();
    /*
    NE::InputManager::ArrowsDirection directions = m_pEngine->getInputManager()->getDirectionsPressed();
    NE::InputManager::Buttons buttons = m_pEngine->getInputManager()->getButtonsPressed();
    */
    IVec2 mousePos;
    int mouseButtons = SDL_GetMouseState(&mousePos.x,&mousePos.y) & SDL_BUTTON(SDL_BUTTON_LEFT);
    
    if ( m_state == Game::GS_TITLE )
    {
        if ( mouseButtons & SDL_BUTTON(SDL_BUTTON_LEFT))
        {
            if ( mousePos.x > WINDOW_WIDTH/2-(m_resLoader.getSprite(NEWGAME_PATH))->getSize().width/2 &&
                 mousePos.x < WINDOW_WIDTH/2+(m_resLoader.getSprite(NEWGAME_PATH))->getSize().width/2 )
            {
                if ( mousePos.y > WINDOW_HEIGHT/2-(m_resLoader.getSprite(NEWGAME_PATH))->getSize().height/2 - 100 &&
                     mousePos.y < WINDOW_HEIGHT/2+(m_resLoader.getSprite(NEWGAME_PATH))->getSize().height/2 - 100 )
                {
                    m_state=GS_GAME;
                    resetGame();
                }
                else if (mousePos.y > WINDOW_HEIGHT/2-(m_resLoader.getSprite(QUIT_PATH))->getSize().height/2 + 100 &&
                         mousePos.y < WINDOW_HEIGHT/2+(m_resLoader.getSprite(QUIT_PATH))->getSize().height/2 + 100 )
                {
                    m_isRunning = false;
                }
            }
        }
    }
    else if ( m_state == Game::GS_GAME )
    {
        for (std::vector<Piece*>::iterator itPiece = m_pieces.begin() ; itPiece != m_pieces.end() ; )
        {
            if ( (*itPiece)->isDead() )
            {
                delete (*itPiece);
                itPiece = m_pieces.erase(itPiece);
                continue;
            }

            if ( !(*itPiece)->isFrozen() && !(*itPiece)->isExplosing() )
            {
                bool frozen = (*itPiece)->fall(m_gameInfo.fallSpeed * deltaTime);
                if ( !frozen )
                {
                    // Check collision with all frozen, to see if we collide
                    for (std::vector<Piece*>::iterator itFrozenPiece = m_pieces.begin() ; itFrozenPiece != m_pieces.end() ;
                         ++itFrozenPiece)
                    {
                        if ( (*itFrozenPiece)->isFrozen() )
                        {
                            if ( (*itFrozenPiece)->collides(*(*itPiece)) )
                            {
                                (*itPiece)->freeze();
                                if ( (*itPiece)->isGameOver() )
                                {
                                    m_state = GS_GAMEOVER;
                                }
                                break;
                            }
                        }
                    }
                }
            }
            ++itPiece;
        }

        for (std::vector<Bomb*>::iterator itBomb = m_bombs.begin() ; itBomb != m_bombs.end() ; )
        {
            bool hasExploded = (*itBomb)->isExplosing();

            (*itBomb)->update(m_gameInfo.bombSpeed * deltaTime);

            if ( !hasExploded && (*itBomb)->isExplosing() )
            {
                for (std::vector<Piece*>::iterator itPiece = m_pieces.begin() ; itPiece != m_pieces.end() ; ++itPiece )
                {
                    if ( (*itPiece)->isImpacted((*itBomb)->explosionPosition()) )
                    {
                        (*itPiece)->explode();
                        m_gameInfo.nbPiecesKilled++;
                        m_gameInfo.nbBlocksKilled+=(*itPiece)->nbBlocks();
                        m_gameInfo.score+=55*m_gameInfo.level;
                        if ( m_gameInfo.nbPiecesKilled % 10 == 0 && m_gameInfo.nbPiecesKilled != 0 )
                        {
                            this->levelUp();
                        }
                    }
                }
            }

            if ( (*itBomb)->isExploded() )
            {
                delete (*itBomb);
                itBomb = m_bombs.erase(itBomb);
            }
            else
            {
                ++itBomb;
            }
        }

        if ( currentTime - m_lastDropTime > m_gameInfo.dropDelay )
        {
            spawnPiece();
            m_lastDropTime = currentTime;
        }

        if ( mouseButtons & SDL_BUTTON(SDL_BUTTON_LEFT))
        {
            if ( mousePos.x > PANEL_LEFT_POSX+8 && mousePos.x < PANEL_RIGHT_POSX &&
                 mousePos.y < WINDOW_HEIGHT-BASE_HEIGHT )
            {
                if ( currentTime - m_lastShootTime > m_gameInfo.shootDelay )
                {
                    if ( m_bombs.size() < m_gameInfo.bombNumber )
                    {
                        spawnBomb(mousePos);
                        m_lastShootTime = currentTime;
                    }
                }
            }
            else if (m_pBoardGame->isBonusTime() )
            {
                if ( mousePos.x > PANEL_RIGHT_POSX+8+58 && mousePos.x < PANEL_RIGHT_POSX+8+140 )
                {
                    if ( mousePos.y > BOMBS_POSY && mousePos.y < BOMBS_POSY + 105 )
                    {
                        m_pBoardGame->setBonusTime(false);
                        m_gameInfo.bombNumber++;
                    }
                    else if ( mousePos.y > SPEED_POSY && mousePos.y < SPEED_POSY + 105 )
                    {
                        m_pBoardGame->setBonusTime(false);
                        m_gameInfo.bombSpeed += m_gameInfo.bombSpeed/3.f;
                    }
                    else if ( mousePos.y > TIME_POSY && mousePos.y < TIME_POSY + 105 )
                    {
                        if ( m_gameInfo.shootDelay > 0 )
                        {
                            m_pBoardGame->setBonusTime(false);
                            m_gameInfo.shootDelay -= 50;
                        }
                    }
                }
            }
        }
    }
    else if ( m_state == GS_GAMEOVER )
    {
        if ( mouseButtons & SDL_BUTTON(SDL_BUTTON_LEFT))
        {
            m_state=GS_TITLE;
        }
    }

    m_pCursor->setPosition(mousePos);

    return true;
}


void Game::resetGame()
{
    m_pBoardGame->setBonusTime(false);
    m_gameInfo.reset();
}

void Game::spawnPiece()
{
    m_pieces.push_back(new Piece(m_pieceGenerator.getPiece(),m_resLoader.getSprite(BLOCK_EXPLOSION_PATH)));
    // NEDebug << "Spawn\n";
}

void Game::spawnBomb(const IVec2& mousePos)
{
    m_bombs.push_back(new Bomb(UVec2(START_BOMB_POSX,START_BOMB_POSY),
                               UVec2(mousePos.x,mousePos.y),
                               m_resLoader.getSprite(BOMB_PATH),
                               m_resLoader.getSprite(EXPLOSION_PATH)));
}

void Game::levelUp()
{
    m_pBoardGame->setBonusTime(true);

    m_gameInfo.dropDelay -= m_gameInfo.dropDelay/8.f;
    m_gameInfo.fallSpeed += m_gameInfo.fallSpeed/3.f;

    m_gameInfo.level++;

    NEDebug << "Level UP : dropDelay : " << m_gameInfo.dropDelay << " Fall : " << m_gameInfo.fallSpeed << "\n";
}
