/**
TetrisCommand is a remake of the eponym game from Atari ST / Amiga
The program is developped under an event that took place on internet with the goal to make a game in a weekend
Copyright (C) 2014  Alexandre "LittleWhite" LAURENT

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

website: http://www.developpez.net/forums/d1465457/applications/developpement-2d-3d-jeux/projets/we-jv4-mix-weekend-tetriscommand/
e-mail: lw.demoscene@gmail.com
**/

#include "Piece.hpp"

#include "NEngine/Sprite.h"
#include "NEngine/Renderer.h"

#include "Engine/AnimatedSprite.h"

Piece::Piece(const std::list<Block>& blocks, const NE::Sprite* pExplositionSprite)
    :m_blocks(blocks),m_pExplosionSprite(new AnimatedSprite(pExplositionSprite,USize2(BLOCK_EXPLOSION_WIDTH,BLOCK_EXPLOSION_HEIGHT),BLOCK_EXPLOSION_ANIMATION_STEP_TIME)),
     m_explositionTime(0)
{

}

Piece::~Piece()
{
    delete m_pExplosionSprite;
}

void Piece::freeze()
{
    for(std::list<Block>::iterator itBlock = m_blocks.begin() ;
        itBlock != m_blocks.end() ;
        ++itBlock )
    {
        itBlock->color=0;
    }
}

bool Piece::fall(float speed)
{
    // Check if we reach the floor
    for(std::list<Block>::iterator itBlock = m_blocks.begin() ;
        itBlock != m_blocks.end() ;
        ++itBlock )
    {
        if ( itBlock->position.y+BLOCK_SIZE/2 > WINDOW_HEIGHT)
        {
            freeze();
            return true;
        }
    }

    for(std::list<Block>::iterator itBlock = m_blocks.begin() ;
        itBlock != m_blocks.end() ;
        ++itBlock )
    {
        itBlock->fall(speed);
    }

    return false;
}

bool Piece::collides(const Piece& other)const
{
    for(std::list<Block>::const_iterator itBlock = m_blocks.begin() ;
        itBlock != m_blocks.end() ;
        ++itBlock )
    {
        for(std::list<Block>::const_iterator itOtherBlock = other.m_blocks.begin() ;
            itOtherBlock != other.m_blocks.end() ;
            ++itOtherBlock )
        {
            if ( itBlock->collides(*itOtherBlock) )
            {
                return true;
            }
        }
    }

    return false;
}

bool Piece::isFrozen()const
{
    for(std::list<Block>::const_iterator itBlock = m_blocks.begin() ;
        itBlock != m_blocks.end() ;
        ++itBlock )
    {
        return itBlock->isFrozen();
    }
    // Should never reach this, but well, remove a warning
    return false;
}

void Piece::explode()
{
    for(std::list<Block>::iterator itBlock = m_blocks.begin() ;
        itBlock != m_blocks.end() ;
        ++itBlock )
    {
        itBlock->explode();
    }
}

bool Piece::render(const NE::Renderer& renderer, const NE::Sprite* pBlocksSprite, const unsigned time)
{
    bool result = true;

    for(std::list<Block>::iterator itBlock = m_blocks.begin() ;
        itBlock != m_blocks.end() ;
        ++itBlock )
    {
        if ( itBlock->isExplosing() )
        {
            if ( m_explositionTime == 0 )
            {
                m_explositionTime = time;
            }
            if ( time - m_explositionTime > BLOCK_EXPLOSION_ANIMATION_STEP_TIME * 6)
            {
                itBlock->die();
                break;
            }

            result &= m_pExplosionSprite->draw(renderer,
                                               IVec2(itBlock->block.position.x-BLOCK_EXPLOSION_WIDTH/2,itBlock->block.position.y-BLOCK_EXPLOSION_HEIGHT/2),
                                               time);
        }
        else if ( itBlock->isDead() )
        {

        }
        else
        {
            result &= renderer.drawSurface(IVec2(itBlock->block.position.x-BLOCK_SIZE/2,itBlock->block.position.y-BLOCK_SIZE/2),pBlocksSprite,
                                           Rect(IVec2(itBlock->color*BLOCK_SIZE,0),USize2(BLOCK_SIZE,BLOCK_SIZE)));
        }
    }

    return result;
}

bool Piece::isExplosing()const
{
    for(std::list<Block>::const_iterator itBlock = m_blocks.begin() ;
        itBlock != m_blocks.end() ;
        ++itBlock )
    {
        if ( itBlock->isExplosing() == false )
        {
            return false;
        }
    }
    return true;
}

bool Piece::isDead()const
{
    for(std::list<Block>::const_iterator itBlock = m_blocks.begin() ;
        itBlock != m_blocks.end() ;
        ++itBlock )
    {
        if ( itBlock->isDead() == false )
        {
            return false;
        }
    }
    return true;
}

bool Piece::isGameOver()const
{
    for(std::list<Block>::const_iterator itBlock = m_blocks.begin() ;
        itBlock != m_blocks.end() ;
        ++itBlock )
    {
        if ( itBlock->position.y < WINDOW_HEIGHT-BASE_HEIGHT)
        {
            return true;
        }
    }

    return false;
}

bool Piece::isImpacted(const UVec2& explosionPosition)
{
    if ( this->isFrozen() )
    {
        return false;
    }

    for(std::list<Block>::const_iterator itBlock = m_blocks.begin() ;
        itBlock != m_blocks.end() ;
        ++itBlock )
    {
        if ( ! itBlock->isExplosing() )
        {
            unsigned dist = distance(explosionPosition, UVec2(itBlock->position.x,itBlock->position.y));
            if ( dist < BOMB_RADIUS_EFFECT )
            {
                return true;
            }
        }
    }
    return false;
}
